@@ -441,8 +441,10 @@ asb_utils_write_archive (const gchar *filename,
 	struct stat st;
 
 	a = archive_write_new ();
-	if (g_str_has_suffix (filename, ".gz"))
+	if (g_str_has_suffix (filename, ".gz")) {
 		archive_write_add_filter_gzip (a);
+		archive_write_set_filter_option (a, "gz", "timestamp", NULL);
+	}
 	if (g_str_has_suffix (filename, ".bz2"))
 		archive_write_add_filter_bzip2 (a);
 	if (g_str_has_suffix (filename, ".xz"))
@@ -506,6 +508,12 @@ asb_utils_add_files_recursive (GPtrArray *files,
 	return TRUE;
 }
 
+static gint
+my_pstrcmp (const gchar **a, const gchar **b)
+{
+	return g_strcmp0 (*a, *b);
+}
+
 /**
  * asb_utils_write_archive_dir:
  * @filename: archive filename
@@ -532,6 +540,9 @@ asb_utils_write_archive_dir (const gchar *filename,
 	if (files->len == 0)
 		return TRUE;
 
+	/* sort by filename for deterministic results */
+	g_ptr_array_sort (files, (GCompareFunc) my_pstrcmp);
+
 	/* write tar file */
 	return asb_utils_write_archive (filename, directory, files, error);
 }
