@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -68,8 +68,8 @@ typedef struct
 	GHashTable		*hash_pkgname;	/* of AsApp{pkgname} */
 	AsMonitor		*monitor;
 	GHashTable		*metadata_indexes;	/* GHashTable{key} */
-	GHashTable		*appinfo_dirs;	/* GHashTable{path} */
-	GHashTable		*xdg_app_dirs;	/* GHashTable{path} */
+	GHashTable		*appinfo_dirs;	/* GHashTable{path:AsStorePathData} */
+	GHashTable		*flatpak_dirs;	/* GHashTable{path:id_prefix} */
 	AsStoreAddFlags		 add_flags;
 	AsStoreWatchFlags	 watch_flags;
 	AsStoreProblems		 problems;
@@ -79,6 +79,11 @@ typedef struct
 	AsProfile		*profile;
 } AsStorePrivate;
 
+typedef struct {
+	gchar			*id_prefix;
+	gchar			*arch;
+} AsStorePathData;
+
 G_DEFINE_TYPE_WITH_PRIVATE (AsStore, as_store, G_TYPE_OBJECT)
 
 enum {
@@ -99,7 +104,7 @@ static guint signals [SIGNAL_LAST] = { 0 };
  **/
 G_DEFINE_QUARK (as-store-error-quark, as_store_error)
 
-static gboolean	as_store_search_xdg_apps (AsStore *store,
+static gboolean	as_store_search_flatpaks (AsStore *store,
 					  AsStoreLoadFlags flags,
 					  const gchar *id_prefix,
 					  const gchar *path,
@@ -108,6 +113,7 @@ static gboolean	as_store_search_xdg_apps (AsStore *store,
 static gboolean	as_store_from_file_internal (AsStore *store,
 					     GFile *file,
 					     const gchar *id_prefix,
+					     const gchar *arch,
 					     GCancellable *cancellable,
 					     GError **error);
 
@@ -130,7 +136,7 @@ as_store_finalize (GObject *object)
 	g_hash_table_unref (priv->hash_pkgname);
 	g_hash_table_unref (priv->metadata_indexes);
 	g_hash_table_unref (priv->appinfo_dirs);
-	g_hash_table_unref (priv->xdg_app_dirs);
+	g_hash_table_unref (priv->flatpak_dirs);
 
 	G_OBJECT_CLASS (as_store_parent_class)->finalize (object);
 }
@@ -574,6 +580,7 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 	} id_map[] = {
 		/* GNOME */
 		{ "baobab.desktop",		"org.gnome.baobab.desktop" },
+		{ "bijiben.desktop",		"org.gnome.bijiben.desktop" },
 		{ "cheese.desktop",		"org.gnome.Cheese.desktop" },
 		{ "devhelp.desktop",		"org.gnome.Devhelp.desktop" },
 		{ "file-roller.desktop",	"org.gnome.FileRoller.desktop" },
@@ -594,6 +601,7 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 		{ "gnome-documents.desktop",	"org.gnome.Documents.desktop" },
 		{ "gnome-font-viewer.desktop",	"org.gnome.font-viewer.desktop" },
 		{ "gnome-maps.desktop",		"org.gnome.Maps.desktop" },
+		{ "gnome-nibbles.desktop",	"org.gnome.Nibbles.desktop" },
 		{ "gnome-photos.desktop",	"org.gnome.Photos.desktop" },
 		{ "gnome-screenshot.desktop",	"org.gnome.Screenshot.desktop" },
 		{ "gnome-software.desktop",	"org.gnome.Software.desktop" },
@@ -604,6 +612,7 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 		{ "gnotravex.desktop",		"gnome-tetravex.desktop" },
 		{ "gnotski.desktop",		"gnome-klotski.desktop" },
 		{ "gtali.desktop",		"tali.desktop" },
+		{ "latexila.desktop",		"org.gnome.latexila.desktop" },
 		{ "nautilus.desktop",		"org.gnome.Nautilus.desktop" },
 		{ "polari.desktop",		"org.gnome.Polari.desktop" },
 		{ "totem.desktop",		"org.gnome.Totem.desktop" },
@@ -666,6 +675,24 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 	return NULL;
 }
 
+/**
+ * as_app_has_pkgname:
+ **/
+static gboolean
+as_app_has_pkgname (AsApp *app, const gchar *pkgname)
+{
+	guint i;
+	GPtrArray *pkgnames;
+
+	pkgnames = as_app_get_pkgnames (app);
+	for (i = 0; i < pkgnames->len; i++) {
+		const gchar *tmp = g_ptr_array_index (pkgnames, i);
+		if (g_strcmp0 (tmp, pkgname) == 0)
+			return TRUE;
+	}
+	return FALSE;
+}
+
 /**
  * as_store_get_app_by_pkgname:
  * @store: a #AsStore instance.
@@ -680,9 +707,27 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 AsApp *
 as_store_get_app_by_pkgname (AsStore *store, const gchar *pkgname)
 {
+	AsApp *app;
 	AsStorePrivate *priv = GET_PRIVATE (store);
+	guint i;
+
 	g_return_val_if_fail (AS_IS_STORE (store), NULL);
-	return g_hash_table_lookup (priv->hash_pkgname, pkgname);
+
+	/* in most cases, we can use the cache */
+	app = g_hash_table_lookup (priv->hash_pkgname, pkgname);
+	if (app != NULL)
+		return app;
+
+	/* fall back in case the user adds to app to the store, *then*
+	 * uses as_app_add_pkgname() on the app */
+	for (i = 0; i < priv->array->len; i++) {
+		app = g_ptr_array_index (priv->array, i);
+		if (as_app_has_pkgname (app, pkgname))
+			return app;
+	}
+
+	/* not found */
+	return NULL;
 }
 
 /**
@@ -931,10 +976,10 @@ as_store_match_addons (AsStore *store)
 }
 
 /**
- * as_store_get_origin_for_xdg_app:
+ * as_store_get_origin_for_flatpak:
  **/
 static gchar *
-as_store_get_origin_for_xdg_app (const gchar *fn)
+as_store_get_origin_for_flatpak (const gchar *fn)
 {
 	g_auto(GStrv) split = g_strsplit (fn, "/", -1);
 	guint chunks = g_strv_length (split);
@@ -950,7 +995,7 @@ as_store_get_origin_for_xdg_app (const gchar *fn)
  * continue to use the application ID as the primary identifier.
  *
  * Now we support installing things per-user, and also per-system and per-user
- * xdg-app (not even including jhbuild) we need to use the id prefix to
+ * flatpak (not even including jhbuild) we need to use the id prefix to
  * disambiguate the different applications according to a 'scope'.
  *
  * This means when we launch a specific application in the software center
@@ -978,20 +1023,29 @@ as_store_from_root (AsStore *store,
 		    const gchar *id_prefix,
 		    const gchar *icon_prefix,
 		    const gchar *source_filename,
+		    const gchar *arch,
 		    GError **error)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	AsNode *apps;
 	AsNode *n;
 	const gchar *tmp;
+	const gchar *origin_delim = ":";
+	gchar *str;
 	g_autofree AsNodeContext *ctx = NULL;
 	g_autofree gchar *icon_path = NULL;
+	g_autofree gchar *id_prefix_app = NULL;
 	g_autofree gchar *origin_app = NULL;
 	_cleanup_uninhibit_ guint32 *tok = NULL;
 	g_autoptr(AsProfileTask) ptask = NULL;
 
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
 
+	/* make throws us under a bus, yet again */
+	tmp = g_getenv ("AS_SELF_TEST_PREFIX_DELIM");
+	if (tmp != NULL)
+		origin_delim = tmp;
+
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:store-from-root");
 
@@ -1021,14 +1075,64 @@ as_store_from_root (AsStore *store,
 	if (tmp != NULL)
 		as_store_set_origin (store, tmp);
 
-	/* special case xdg-apps */
-	if (source_filename != NULL &&
-	    g_strcmp0 (priv->origin, "xdg-app") == 0) {
-		origin_app = as_store_get_origin_for_xdg_app (source_filename);
+	/* origin has prefix already specified in the XML */
+	if (priv->origin != NULL) {
+		str = g_strstr_len (priv->origin, -1, origin_delim);
+		if (str != NULL) {
+			id_prefix_app = g_strdup (priv->origin);
+			str = g_strstr_len (id_prefix_app, -1, origin_delim);
+			if (str != NULL) {
+				str[0] = '\0';
+				origin_app = g_strdup (str + 1);
+			}
+		}
+	}
+
+	/* special case flatpak symlinks -- scope:name.xml.gz */
+	if (origin_app == NULL &&
+	    g_strcmp0 (priv->origin, "flatpak") == 0 &&
+	    source_filename != NULL &&
+	    g_file_test (source_filename, G_FILE_TEST_IS_SYMLINK)) {
+		g_autofree gchar *source_basename = NULL;
+
+		/* get the origin */
+		source_basename = g_path_get_basename (source_filename);
+		str = g_strrstr (source_basename, ".xml");
+		if (str != NULL) {
+			str[0] = '\0';
+			origin_app = g_strdup (source_basename);
+		}
+
+		/* get the id-prefix */
+		str = g_strstr_len (source_basename, -1, origin_delim);
+		if (str != NULL) {
+			str[0] = '\0';
+			id_prefix_app = g_strdup (source_basename);
+		}
+	}
+
+	/* FIXME: we can remove this helper when nuking FLATPAK */
+	if (origin_app == NULL &&
+	    source_filename != NULL &&
+	    g_strcmp0 (priv->origin, "flatpak") == 0) {
+		id_prefix_app = g_strdup (id_prefix);
+		origin_app = as_store_get_origin_for_flatpak (source_filename);
+	}
+
+	/* fallback */
+	if (origin_app == NULL) {
+		id_prefix_app = g_strdup (id_prefix);
+		origin_app = g_strdup (priv->origin);
+	}
+
+	/* print what cleverness we did */
+	if (g_strcmp0 (origin_app, priv->origin) != 0) {
 		g_debug ("using app origin of '%s' rather than '%s'",
 			 origin_app, priv->origin);
-	} else {
-		origin_app = g_strdup (priv->origin);
+	}
+	if (g_strcmp0 (id_prefix_app, id_prefix) != 0) {
+		g_debug ("using app prefix of '%s' rather than '%s'",
+			 id_prefix_app, id_prefix);
 	}
 
 	/* guess the icon path after we've read the origin and then look for
@@ -1038,12 +1142,12 @@ as_store_from_root (AsStore *store,
 		topdir = g_path_get_basename (icon_prefix);
 		if ((g_strcmp0 (topdir, "xmls") == 0 ||
 		     g_strcmp0 (topdir, "yaml") == 0)
-		    && priv->origin != NULL) {
+		    && origin_app != NULL) {
 			g_autofree gchar *dirname = NULL;
 			dirname = g_path_get_dirname (icon_prefix);
 			icon_path = g_build_filename (dirname,
 						      "icons",
-						      priv->origin,
+						      origin_app,
 						      NULL);
 		} else {
 			icon_path = g_build_filename (icon_prefix, "icons", NULL);
@@ -1077,6 +1181,8 @@ as_store_from_root (AsStore *store,
 		app = as_app_new ();
 		if (icon_path != NULL)
 			as_app_set_icon_path (app, icon_path);
+		if (arch != NULL)
+			as_app_add_arch (app, arch);
 		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPSTREAM);
 		if (!as_app_node_parse (app, n, ctx, &error_local)) {
 			g_set_error (error,
@@ -1088,7 +1194,7 @@ as_store_from_root (AsStore *store,
 		}
 
 		/* set the correct scope */
-		as_store_fixup_id_prefix (app, id_prefix);
+		as_store_fixup_id_prefix (app, id_prefix_app);
 
 		if (origin_app != NULL)
 			as_app_set_origin (app, origin_app);
@@ -1197,10 +1303,10 @@ as_store_load_yaml_file (AsStore *store,
 }
 
 /**
- * as_store_rescan_xdg_app_dir:
+ * as_store_rescan_flatpak_dir:
  */
 static void
-as_store_rescan_xdg_app_dir (AsStore *store, const gchar *filename)
+as_store_rescan_flatpak_dir (AsStore *store, const gchar *filename)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	const gchar *id_prefix;
@@ -1208,13 +1314,13 @@ as_store_rescan_xdg_app_dir (AsStore *store, const gchar *filename)
 	g_autoptr(GError) error = NULL;
 
 	/* we helpfully saved this */
-	id_prefix = g_hash_table_lookup (priv->xdg_app_dirs, filename);
+	id_prefix = g_hash_table_lookup (priv->flatpak_dirs, filename);
 
-	g_debug ("rescanning xdg-app dir %s", filename);
+	g_debug ("rescanning flatpak dir %s", filename);
 	dest = g_build_filename (priv->destdir ? priv->destdir : "/", filename, NULL);
 	if (!g_file_test (dest, G_FILE_TEST_EXISTS))
 		return;
-	if (!as_store_search_xdg_apps (store,
+	if (!as_store_search_flatpaks (store,
 				       AS_STORE_LOAD_FLAG_IGNORE_INVALID,
 				       id_prefix,
 				       dest,
@@ -1234,6 +1340,7 @@ as_store_remove_by_source_file (AsStore *store, const gchar *filename)
 	GPtrArray *apps;
 	guint i;
 	const gchar *tmp;
+	_cleanup_uninhibit_ guint32 *tok = NULL;
 	g_autoptr(GPtrArray) ids = NULL;
 
 	/* find any applications in the store with this source file */
@@ -1247,6 +1354,7 @@ as_store_remove_by_source_file (AsStore *store, const gchar *filename)
 	}
 
 	/* remove these from the store */
+	tok = as_store_changed_inhibit (store);
 	for (i = 0; i < ids->len; i++) {
 		tmp = g_ptr_array_index (ids, i);
 		g_debug ("removing %s as %s invalid", tmp, filename);
@@ -1254,9 +1362,57 @@ as_store_remove_by_source_file (AsStore *store, const gchar *filename)
 	}
 
 	/* the store changed */
+	as_store_changed_uninhibit (&tok);
 	as_store_perhaps_emit_changed (store, "remove-by-source-file");
 }
 
+/**
+ * as_store_watch_source_added:
+ */
+static void
+as_store_watch_source_added (AsStore *store, const gchar *filename)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	AsStorePathData *path_data;
+	g_autofree gchar *dirname = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
+
+	/* ignore directories */
+	if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR))
+		return;
+
+	/* we helpfully saved this */
+	dirname = g_path_get_dirname (filename);
+	g_debug ("parsing new file %s from %s", filename, dirname);
+	path_data = g_hash_table_lookup (priv->appinfo_dirs, dirname);
+	if (path_data == NULL) {
+		g_warning ("no path data for %s", dirname);
+		return;
+	}
+	file = g_file_new_for_path (filename);
+	if (!as_store_from_file_internal (store,
+					  file,
+					  path_data->id_prefix,
+					  path_data->arch,
+					  NULL, /* cancellable */
+					  &error)){
+		g_warning ("failed to rescan: %s", error->message);
+	}
+}
+
+/**
+ * as_store_watch_source_changed:
+ */
+static void
+as_store_watch_source_changed (AsStore *store, const gchar *filename)
+{
+	/* remove all the apps provided by the source file then re-add them */
+	g_debug ("re-parsing changed file %s", filename);
+	as_store_remove_by_source_file (store, filename);
+	as_store_watch_source_added (store, filename);
+}
+
 /**
  * as_store_monitor_changed_cb:
  */
@@ -1266,33 +1422,16 @@ as_store_monitor_changed_cb (AsMonitor *monitor,
 			     AsStore *store)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
+	_cleanup_uninhibit_ guint32 *tok = NULL;
 
 	/* reload, or emit a signal */
+	tok = as_store_changed_inhibit (store);
 	if (priv->watch_flags & AS_STORE_WATCH_FLAG_ADDED) {
-		_cleanup_uninhibit_ guint32 *tok = NULL;
-		tok = as_store_changed_inhibit (store);
-
-		if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
-			const gchar *id_prefix;
-			g_autoptr(GError) error = NULL;
-			g_autoptr(GFile) file = NULL;
-			as_store_remove_by_source_file (store, filename);
-			g_debug ("rescanning %s", filename);
-			file = g_file_new_for_path (filename);
-
-			/* we helpfully saved this */
-			id_prefix = g_hash_table_lookup (priv->appinfo_dirs, filename);
-			if (!as_store_from_file_internal (store,
-							  file,
-							  id_prefix,
-							  NULL,
-							  &error)){
-				g_warning ("failed to rescan: %s", error->message);
-			}
-		} else if (g_hash_table_contains (priv->xdg_app_dirs, filename)) {
-			as_store_rescan_xdg_app_dir (store, filename);
-		}
+		as_store_watch_source_changed (store, filename);
+		if (g_hash_table_contains (priv->flatpak_dirs, filename))
+			as_store_rescan_flatpak_dir (store, filename);
 	}
+	as_store_changed_uninhibit (&tok);
 	as_store_perhaps_emit_changed (store, "file changed");
 }
 
@@ -1305,24 +1444,16 @@ as_store_monitor_added_cb (AsMonitor *monitor,
 			     AsStore *store)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
+	_cleanup_uninhibit_ guint32 *tok = NULL;
 
 	/* reload, or emit a signal */
+	tok = as_store_changed_inhibit (store);
 	if (priv->watch_flags & AS_STORE_WATCH_FLAG_ADDED) {
-		_cleanup_uninhibit_ guint32 *tok = NULL;
-		tok = as_store_changed_inhibit (store);
-		if (g_file_test (filename, G_FILE_TEST_IS_REGULAR)) {
-			g_autoptr(GError) error = NULL;
-			g_autoptr(GFile) file = NULL;
-
-			g_debug ("scanning %s", filename);
-			file = g_file_new_for_path (filename);
-			if (!as_store_from_file (store, file, NULL, NULL, &error)) //FIXME: id_kind
-				g_warning ("failed to rescan: %s", error->message);
-		} else if (g_hash_table_contains (priv->xdg_app_dirs, filename)) {
-			as_store_rescan_xdg_app_dir (store, filename);
-		}
-
+		as_store_watch_source_added (store, filename);
+		if (g_hash_table_contains (priv->flatpak_dirs, filename))
+			as_store_rescan_flatpak_dir (store, filename);
 	}
+	as_store_changed_uninhibit (&tok);
 	as_store_perhaps_emit_changed (store, "file added");
 }
 
@@ -1343,6 +1474,50 @@ as_store_monitor_removed_cb (AsMonitor *monitor,
 	}
 }
 
+/**
+ * as_store_add_path_data:
+ *
+ * Save the path data so we can add any newly-discovered applications with the
+ * correct prefix and architecture.
+ **/
+static void
+as_store_add_path_data (AsStore *store,
+			const gchar *path,
+			const gchar *id_prefix,
+			const gchar *arch)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	AsStorePathData *path_data;
+
+	/* check is a directory */
+	if (!g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		g_warning ("not adding path %s [%s:%s] as not a directory",
+			   path, id_prefix, arch);
+		return;
+	}
+
+	/* check not already exists */
+	path_data = g_hash_table_lookup (priv->appinfo_dirs, path);
+	if (path_data != NULL) {
+		if (g_strcmp0 (path_data->id_prefix, id_prefix) != 0 ||
+		    g_strcmp0 (path_data->arch, arch) != 0) {
+			g_warning ("already added path %s [%s:%s] vs new [%s:%s]",
+				   path, path_data->id_prefix, path_data->arch,
+				   id_prefix, arch);
+		} else {
+			g_debug ("already added path %s [%s:%s]",
+				 path, path_data->id_prefix, path_data->arch);
+		}
+		return;
+	}
+
+	/* create new */
+	path_data = g_slice_new0 (AsStorePathData);
+	path_data->id_prefix = g_strdup (id_prefix);
+	path_data->arch = g_strdup (arch);
+	g_hash_table_insert (priv->appinfo_dirs, g_strdup (path), path_data);
+}
+
 /**
  * as_store_from_file_internal:
  **/
@@ -1350,6 +1525,7 @@ static gboolean
 as_store_from_file_internal (AsStore *store,
 			     GFile *file,
 			     const gchar *id_prefix,
+			     const gchar *arch,
 			     GCancellable *cancellable,
 			     GError **error)
 {
@@ -1402,7 +1578,7 @@ as_store_from_file_internal (AsStore *store,
 	/* icon prefix is the directory the XML has been found in */
 	icon_prefix = g_path_get_dirname (filename);
 	return as_store_from_root (store, root, id_prefix,
-				   icon_prefix, filename, error);
+				   icon_prefix, filename, arch, error);
 }
 
 /**
@@ -1431,7 +1607,10 @@ as_store_from_file (AsStore *store,
 		    GCancellable *cancellable,
 		    GError **error)
 {
-	return as_store_from_file_internal (store, file, NULL, cancellable, error);
+	return as_store_from_file_internal (store, file,
+					    NULL, /* id_prefix */
+					    NULL, /* arch */
+					    cancellable, error);
 }
 
 /**
@@ -1515,7 +1694,6 @@ as_store_from_xml (AsStore *store,
 {
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(AsNode) root = NULL;
-	const gchar *id_prefix = NULL;
 
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
 	g_return_val_if_fail (data != NULL, FALSE);
@@ -1535,7 +1713,12 @@ as_store_from_xml (AsStore *store,
 			     error_local->message);
 		return FALSE;
 	}
-	return as_store_from_root (store, root, id_prefix, icon_root, NULL, error);
+	return as_store_from_root (store, root,
+				   NULL, /* id_prefix */
+				   icon_root,
+				   NULL, /* filename */
+				   NULL, /* arch */
+				   error);
 }
 
 /**
@@ -2050,6 +2233,7 @@ static gboolean
 as_store_load_app_info_file (AsStore *store,
 			     const gchar *id_prefix,
 			     const gchar *path_xml,
+			     const gchar *arch,
 			     GCancellable *cancellable,
 			     GError **error)
 {
@@ -2068,6 +2252,7 @@ as_store_load_app_info_file (AsStore *store,
 	return as_store_from_file_internal (store,
 					    file,
 					    id_prefix,
+					    arch,
 					    cancellable,
 					    error);
 }
@@ -2079,6 +2264,7 @@ static gboolean
 as_store_load_app_info (AsStore *store,
 			const gchar *id_prefix,
 			const gchar *path,
+			const gchar *arch,
 			AsStoreLoadFlags flags,
 			GCancellable *cancellable,
 			GError **error)
@@ -2101,6 +2287,11 @@ as_store_load_app_info (AsStore *store,
 		return TRUE;
 	dir = g_dir_open (path, 0, &error_local);
 	if (dir == NULL) {
+		if (flags & AS_STORE_LOAD_FLAG_IGNORE_INVALID) {
+			g_warning ("ignoring invalid AppStream path %s: %s",
+				   path, error_local->message);
+			return TRUE;
+		}
 		g_set_error (error,
 			     AS_STORE_ERROR,
 			     AS_STORE_ERROR_FAILED,
@@ -2118,6 +2309,7 @@ as_store_load_app_info (AsStore *store,
 		if (!as_store_load_app_info_file (store,
 						  id_prefix,
 						  filename_md,
+						  arch,
 						  cancellable,
 						  &error_store)) {
 			if (flags & AS_STORE_LOAD_FLAG_IGNORE_INVALID) {
@@ -2132,17 +2324,13 @@ as_store_load_app_info (AsStore *store,
 	}
 
 	/* watch the directories for changes */
+	as_store_add_path_data (store, path, id_prefix, arch);
 	if (!as_monitor_add_directory (priv->monitor,
 				       path,
 				       cancellable,
 				       error))
 		return FALSE;
 
-	/* save the id_prefix */
-	g_hash_table_insert (priv->appinfo_dirs,
-			     g_strdup (path),
-			     g_strdup (id_prefix));
-
 	/* emit changed */
 	as_store_changed_uninhibit (&tok);
 	as_store_perhaps_emit_changed (store, "load-app-info");
@@ -2171,6 +2359,22 @@ as_store_set_app_installed (AsApp *app)
 	as_app_set_state (app, AS_APP_STATE_INSTALLED);
 }
 
+/**
+ * as_store_load_installed_file_is_valid:
+ **/
+static gboolean
+as_store_load_installed_file_is_valid (const gchar *filename)
+{
+	if (g_str_has_suffix (filename, ".desktop"))
+		return TRUE;
+	if (g_str_has_suffix (filename, ".metainfo.xml"))
+		return TRUE;
+	if (g_str_has_suffix (filename, ".appdata.xml"))
+		return TRUE;
+	g_debug ("ignoring filename with invalid suffix: %s", filename);
+	return FALSE;
+}
+
 /**
  * as_store_load_installed:
  **/
@@ -2198,6 +2402,7 @@ as_store_load_installed (AsStore *store,
 		return FALSE;
 
 	/* watch the directories for changes */
+	as_store_add_path_data (store, path, id_prefix, NULL);
 	if (!as_monitor_add_directory (priv->monitor,
 				       path,
 				       cancellable,
@@ -2216,7 +2421,7 @@ as_store_load_installed (AsStore *store,
 		g_autofree gchar *filename = NULL;
 		g_autoptr(AsApp) app = NULL;
 		filename = g_build_filename (path, tmp, NULL);
-		if (!g_file_test (filename, G_FILE_TEST_IS_REGULAR))
+		if (!as_store_load_installed_file_is_valid (filename))
 			continue;
 		if ((priv->add_flags & AS_STORE_ADD_FLAG_PREFER_LOCAL) == 0) {
 			app_tmp = as_store_get_app_by_id (store, tmp);
@@ -2301,6 +2506,7 @@ as_store_search_installed (AsStore *store,
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	g_autofree gchar *dest = NULL;
 	dest = g_build_filename (priv->destdir ? priv->destdir : "/", path, NULL);
+	g_debug ("searching path %s", dest);
 	if (!g_file_test (dest, G_FILE_TEST_EXISTS))
 		return TRUE;
 	return as_store_load_installed (store, flags, id_prefix,
@@ -2330,7 +2536,7 @@ as_store_search_app_info (AsStore *store,
 					 NULL);
 		if (!g_file_test (dest, G_FILE_TEST_EXISTS))
 			continue;
-		if (!as_store_load_app_info (store, id_prefix, dest,
+		if (!as_store_load_app_info (store, id_prefix, dest, NULL,
 					     flags, cancellable, error))
 			return FALSE;
 	}
@@ -2338,10 +2544,10 @@ as_store_search_app_info (AsStore *store,
 }
 
 /**
- * as_store_monitor_xdg_app_dir:
+ * as_store_monitor_flatpak_dir:
  **/
 static void
-as_store_monitor_xdg_app_dir (AsStore *store,
+as_store_monitor_flatpak_dir (AsStore *store,
 			      const gchar *path,
 			      const gchar *id_prefix)
 {
@@ -2349,7 +2555,8 @@ as_store_monitor_xdg_app_dir (AsStore *store,
 	g_autoptr(GError) error = NULL;
 
 	/* mark and monitor directory so we can pick up later added remotes */
-	g_hash_table_insert (priv->xdg_app_dirs, g_strdup (path), g_strdup (id_prefix));
+	g_hash_table_insert (priv->flatpak_dirs, g_strdup (path), g_strdup (id_prefix));
+	as_store_add_path_data (store, path, id_prefix, NULL);
 	if (!as_monitor_add_file (priv->monitor,
 				  path, NULL, &error)) {
 		g_warning ("Can't monitor dir %s: %s",
@@ -2357,85 +2564,62 @@ as_store_monitor_xdg_app_dir (AsStore *store,
 	}
 }
 
-/* until exported by xdg-app */
-#include <sys/utsname.h>
-#include <string.h>
-static const char *
-_xdg_app_get_arch (void)
+/**
+ * as_store_search_flatpak_arch:
+ **/
+static gboolean
+as_store_search_flatpak_arch (AsStore *store,
+			      AsStoreLoadFlags flags,
+			      const gchar *id_prefix,
+			      const gchar *path,
+			      GCancellable *cancellable,
+			      GError **error)
 {
-  static struct utsname buf;
-  static const char *arch = NULL;
-  char *m;
+	const gchar *fn;
+	g_autoptr(GDir) dir = NULL;
 
-  if (arch != NULL)
-    return arch;
-
-  if (uname (&buf))
-    {
-      arch = "unknown";
-      return arch;
-    }
-
-  /* By default, just pass on machine, good enough for most arches */
-  arch = buf.machine;
-
-  /* Override for some arches */
-
-  m = buf.machine;
-  /* i?86 */
-  if (strlen (m) == 4 && m[0] == 'i' && m[2] == '8'  && m[3] == '6')
-    arch = "i386";
-  else if (g_str_has_prefix (m, "arm"))
-    {
-      if (g_str_has_suffix (m, "b"))
-        arch = "armeb";
-      else
-        arch = "arm";
-    }
-  else if (strcmp (m, "mips") == 0)
-    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-      arch = "mipsel";
-#endif
-    }
-  else if (strcmp (m, "mips64") == 0)
-    {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-      arch = "mips64el";
-#endif
-    }
-
-  return arch;
+	dir = g_dir_open (path, 0, NULL);
+	if (dir == NULL)
+		return TRUE;
+	while ((fn = g_dir_read_name (dir)) != NULL) {
+		g_autofree gchar *dest = NULL;
+		dest = g_build_filename (path, fn, "active", NULL);
+		g_debug ("searching path %s", dest);
+		if (!g_file_test (dest, G_FILE_TEST_EXISTS))
+			continue;
+		if (!as_store_load_app_info (store, id_prefix, dest, fn,
+					     flags | AS_STORE_LOAD_FLAG_IGNORE_INVALID,
+					     cancellable, error))
+			return FALSE;
+	}
+	return TRUE;
 }
 
 /**
- * as_store_search_xdg_apps:
+ * as_store_search_flatpaks:
  **/
 static gboolean
-as_store_search_xdg_apps (AsStore *store,
+as_store_search_flatpaks (AsStore *store,
 			  AsStoreLoadFlags flags,
 			  const gchar *id_prefix,
 			  const gchar *path,
 			  GCancellable *cancellable,
 			  GError **error)
 {
-	const gchar *filename;
+	const gchar *fn;
 	g_autoptr(GDir) dir = NULL;
 
 	dir = g_dir_open (path, 0, NULL);
 	if (dir == NULL)
 		return TRUE;
-	while ((filename = g_dir_read_name (dir)) != NULL) {
+	while ((fn = g_dir_read_name (dir)) != NULL) {
 		g_autofree gchar *dest = NULL;
-		dest = g_build_filename (path,
-					 filename,
-					 _xdg_app_get_arch (),
-					 "active",
-					 NULL);
+		dest = g_build_filename (path, fn, NULL);
+		g_debug ("searching path for architectures: %s", dest);
 		if (!g_file_test (dest, G_FILE_TEST_EXISTS))
 			continue;
-		if (!as_store_load_app_info (store, id_prefix, dest,
-					     flags, cancellable, error))
+		if (!as_store_search_flatpak_arch (store, flags, id_prefix,
+						   dest, cancellable, error))
 			return FALSE;
 	}
 	return TRUE;
@@ -2458,49 +2642,57 @@ as_store_search_per_system (AsStore *store,
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:load{per-system}");
 
-	/* xdg-app */
-	if ((flags & AS_STORE_LOAD_FLAG_XDG_APP_SYSTEM) > 0) {
+	/* flatpak */
+	if ((flags & AS_STORE_LOAD_FLAG_FLATPAK_SYSTEM) > 0) {
 		if ((flags & AS_STORE_LOAD_FLAG_APPDATA) > 0) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (LOCALSTATEDIR,
-						 "xdg-app",
+						 "flatpak",
 						 "exports",
 						 "share",
 						 "applications",
 						 NULL);
-			if (!as_store_search_installed (store, flags, "system-xdgapp",
+			if (!as_store_search_installed (store, flags, "flatpak",
 							dest, cancellable, error))
 				return FALSE;
 		}
 		if ((flags & AS_STORE_LOAD_FLAG_DESKTOP) > 0) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (LOCALSTATEDIR,
-						 "xdg-app",
+						 "flatpak",
 						 "exports",
 						 "share",
 						 "appdata",
 						 NULL);
-			if (!as_store_search_installed (store, flags, "system-xdgapp",
+			if (!as_store_search_installed (store, flags, "flatpak",
 							dest, cancellable, error))
 				return FALSE;
 		}
 		if (TRUE) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (LOCALSTATEDIR,
-						 "xdg-app",
+						 "lib",
+						 "flatpak",
 						 "appstream",
 						 NULL);
-			if (!as_store_search_xdg_apps (store, flags, "system-xdgapp",
+			if (!as_store_search_flatpaks (store, flags, "flatpak",
 						       dest, cancellable, error))
 				return FALSE;
-			as_store_monitor_xdg_app_dir (store, dest, "system-xdgapp");
+			as_store_monitor_flatpak_dir (store, dest, "flatpak");
+		}
+		if (g_strcmp0 (LOCALSTATEDIR, "/var") != 0) {
+			const gchar *dest = "/var/lib/flatpak/appstream";
+			if (!as_store_search_flatpaks (store, flags, "flatpak",
+						       dest, cancellable, error))
+				return FALSE;
+			as_store_monitor_flatpak_dir (store, dest, "flatpak");
 		}
 	}
 
 	/* datadir AppStream, AppData and desktop */
 	data_dirs = g_get_system_data_dirs ();
 	for (i = 0; data_dirs[i] != NULL; i++) {
-		if (g_strstr_len (data_dirs[i], -1, "xdg-app/exports") != NULL) {
+		if (g_strstr_len (data_dirs[i], -1, "flatpak/exports") != NULL) {
 			g_debug ("skipping %s as invalid", data_dirs[i]);
 			continue;
 		}
@@ -2518,6 +2710,13 @@ as_store_search_per_system (AsStore *store,
 							dest, cancellable, error))
 				return FALSE;
 		}
+		if ((flags & AS_STORE_LOAD_FLAG_APPDATA) > 0) {
+			g_autofree gchar *dest = NULL;
+			dest = g_build_filename (data_dirs[i], "metainfo", NULL);
+			if (!as_store_search_installed (store, flags, "system",
+							dest, cancellable, error))
+				return FALSE;
+		}
 		if ((flags & AS_STORE_LOAD_FLAG_DESKTOP) > 0) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (data_dirs[i], "applications", NULL);
@@ -2572,36 +2771,36 @@ as_store_search_per_user (AsStore *store,
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:load{per-user}");
 
-	/* xdg-app */
-	if ((flags & AS_STORE_LOAD_FLAG_XDG_APP_USER) > 0) {
+	/* flatpak */
+	if ((flags & AS_STORE_LOAD_FLAG_FLATPAK_USER) > 0) {
 
 		/* installed AppData and desktop */
 		if ((flags & AS_STORE_LOAD_FLAG_APPDATA) > 0) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (g_get_user_data_dir (),
-						 "xdg-app",
+						 "flatpak",
 						 "exports",
 						 "share",
 						 "applications",
 						 NULL);
-			if (!as_store_search_installed (store, flags, "user-xdgapp",
+			if (!as_store_search_installed (store, flags, "user-flatpak",
 							dest, cancellable, error))
 				return FALSE;
 		}
 		if ((flags & AS_STORE_LOAD_FLAG_DESKTOP) > 0) {
 			g_autofree gchar *dest = NULL;
 			dest = g_build_filename (g_get_user_data_dir (),
-						 "xdg-app",
+						 "flatpak",
 						 "exports",
 						 "share",
 						 "appdata",
 						 NULL);
-			if (!as_store_search_installed (store, flags, "user-xdgapp",
+			if (!as_store_search_installed (store, flags, "user-flatpak",
 							dest, cancellable, error))
 				return FALSE;
 		}
 
-		/* If we're running INSIDE the xdg-app environment we'll have the
+		/* If we're running INSIDE the flatpak environment we'll have the
 		 * env var XDG_DATA_HOME set to "~/.var/app/org.gnome.Software/data"
 		 * so specify the path manually to get the real data */
 		if (TRUE) {
@@ -2609,13 +2808,13 @@ as_store_search_per_user (AsStore *store,
 			dest = g_build_filename (g_get_home_dir (),
 						 ".local",
 						 "share",
-						 "xdg-app",
+						 "flatpak",
 						 "appstream",
 						 NULL);
-			if (!as_store_search_xdg_apps (store, flags, "user-xdgapp",
+			if (!as_store_search_flatpaks (store, flags, "user-flatpak",
 						       dest, cancellable, error))
 				return FALSE;
-			as_store_monitor_xdg_app_dir (store, dest, "user-xdgapp");
+			as_store_monitor_flatpak_dir (store, dest, "user-flatpak");
 		}
 	}
 
@@ -2637,6 +2836,15 @@ as_store_search_per_user (AsStore *store,
 			return FALSE;
 	}
 
+	/* MetaInfo */
+	if ((flags & AS_STORE_LOAD_FLAG_APPDATA) > 0) {
+		g_autofree gchar *dest = NULL;
+		dest = g_build_filename (g_get_user_data_dir (), "metainfo", NULL);
+		if (!as_store_search_installed (store, flags, "user",
+						dest, cancellable, error))
+			return FALSE;
+	}
+
 	/* desktop files */
 	if ((flags & AS_STORE_LOAD_FLAG_DESKTOP) > 0) {
 		g_autofree gchar *dest = NULL;
@@ -2732,8 +2940,12 @@ as_store_validate_add (GPtrArray *problems, AsProblemKind kind, const gchar *fmt
 static gchar *
 as_store_get_unique_name_app_key (AsApp *app)
 {
+	const gchar *name;
 	g_autofree gchar *name_lower = NULL;
-	name_lower = g_utf8_strdown (as_app_get_name (app, NULL), -1);
+	name = as_app_get_name (app, NULL);
+	if (name == NULL)
+		return NULL;
+	name_lower = g_utf8_strdown (name, -1);
 	return g_strdup_printf ("<%s:%s>",
 				as_app_kind_to_string (as_app_get_kind (app)),
 				name_lower);
@@ -2891,9 +3103,10 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 			if ((as_app_get_problems (app) & AS_APP_PROBLEM_PREFORMATTED_DESCRIPTION) != 0) {
 				as_store_validate_add (probs,
 						       AS_PROBLEM_KIND_TAG_INVALID,
-						       "metadata version is v%.1f and "
+						       "%s: metadata version is v%.1f and "
 						       "<description> requiring markup "
 						       "was introduced in v0.6",
+						       as_app_get_id (app),
 						       priv->api_version);
 			}
 		}
@@ -2954,27 +3167,41 @@ as_store_validate (AsStore *store, AsAppValidateFlags flags, GError **error)
 		}
 
 		/* check uniqueness */
-		app_key = as_store_get_unique_name_app_key (app);
-		app_tmp = g_hash_table_lookup (hash_names, app_key);
-		if (app_tmp != NULL) {
-			as_store_validate_add (probs,
-					       AS_PROBLEM_KIND_DUPLICATE_DATA,
-					       "%s[%s] as the same name as %s[%s]: %s",
-					       as_app_get_id (app),
-					       as_app_get_pkgname_default (app),
-					       as_app_get_id (app_tmp),
-					       as_app_get_pkgname_default (app_tmp),
-					       app_key);
-		} else {
-			g_hash_table_insert (hash_names,
-					     g_strdup (app_key),
-					     g_object_ref (app));
+		if (as_app_get_kind (app) != AS_APP_KIND_ADDON) {
+			app_key = as_store_get_unique_name_app_key (app);
+			if (app_key != NULL) {
+				app_tmp = g_hash_table_lookup (hash_names, app_key);
+				if (app_tmp != NULL) {
+					as_store_validate_add (probs,
+							       AS_PROBLEM_KIND_DUPLICATE_DATA,
+							       "%s[%s] as the same name as %s[%s]: %s",
+							       as_app_get_id (app),
+							       as_app_get_pkgname_default (app),
+							       as_app_get_id (app_tmp),
+							       as_app_get_pkgname_default (app_tmp),
+							       app_key);
+				} else {
+					g_hash_table_insert (hash_names,
+							     g_strdup (app_key),
+							     g_object_ref (app));
+				}
+			}
 		}
-
 	}
 	return probs;
 }
 
+/**
+ * as_store_path_data_free:
+ **/
+static void
+as_store_path_data_free (AsStorePathData *path_data)
+{
+	g_free (path_data->arch);
+	g_free (path_data->id_prefix);
+	g_slice_free (AsStorePathData, path_data);
+}
+
 /**
  * as_store_init:
  **/
@@ -2997,8 +3224,8 @@ as_store_init (AsStore *store)
 	priv->appinfo_dirs = g_hash_table_new_full (g_str_hash,
 						    g_str_equal,
 						    g_free,
-						    g_free);
-	priv->xdg_app_dirs = g_hash_table_new_full (g_str_hash,
+						    (GDestroyNotify) as_store_path_data_free);
+	priv->flatpak_dirs = g_hash_table_new_full (g_str_hash,
 						    g_str_equal,
 						    g_free,
 						    g_free);
